--[[
QUEUE ENGINE 0.5 ~ Thug.lua

(C) 2013 Clinton S. Cañalita and others
]]--
Thug = 	{}
Thug.__index = Thug

local thex
local dist
local nextX
local nextY
local willhit = false
local thug_walk = 100
local thug_chase = 350
local random_wait_time = math.random(3,5)

function Thug.create(x,y)
	--print("this y = "..y)
	local self = {}
	setmetatable(self, Thug)

	self.alive = true
	self.x = x
	self.y = y
	self.x_vel = 0
	self.y_vel = 0
	self.max_x_vel = thug_walk
	self.max_y_vel = 1000
	self.initx = x
	self.inity = y
	self.h = 32
	self.w = 16
	self.lb = 128 -- 32*4
	self.rb = 80 -- 32*2.5
	self.health = 30
	self.level = 2
	self.def = 5
	self.str = 5
	self.cha = 2
	self.time = 0
	self.dir = 1 -- 0 = left, 1 = right
	self.jump = 40
	self.state = 1 -- 0 = idle, 1 = patrol, 2 = follow, 3 = attack
	self.critical = false -- for animation
	
	self.towards = false
	self.isPush = true
	self.standing = true
	self.is_hit = false
	self.will_hit = false

	--print("thug made")
	return self
end

function Thug:update(dt)
	dist = self:curDist(p.x, p.y, self.x, self.y)
	thex = math.floor(self.x)

	if self.alive then -- makes sure player is alive
			
		self.time = self.time + dt*2 -- make self.time into a mini timer.
		
		-- update thug state
		self:updateState (dt)
		
		-- update thug animation
		self:updateAnim (dt)
	
		  
	  --player is hitting thug
		if dist <= 32 and p.will_hit == true then
			-- the thug is hit
			self.is_hit = true
			-- player attacks!
			
			print("HIT")
			-- calculate damage
			attackDamage(p, self)
			
			-- damage is done
			self.is_hit = false
			p.will_hit = false
		end
	end
	
	if self.health <= 0 then
		self.alive = false
	end
end

function Thug:draw()
  if self.alive then
		if (self.state == 0) then
			tstand:draw(timage, (self.x-16), (self.y-16))
		elseif (self.state == 1) and (self.dir == 1) then
			twalkr:draw(timage, (self.x-16), (self.y-16))
		elseif (self.state == 1) and (self.dir == 0) then
			twalkl:draw(timage, (self.x-16), (self.y-16))
		elseif (self.state == 2) and (self.dir == 1) then
			twalkr:draw(timage, (self.x-16), (self.y-16))
		elseif (self.state == 2) and (self.dir == 0) then
			twalkl:draw(timage, (self.x-16), (self.y-16))
		elseif (self.state == 3) and (self.dir == 1) then
			twalkr:draw(timage, (self.x-16), (self.y-16))
		elseif (self.state == 3) and (self.dir == 0) then
			twalkl:draw(timage, (self.x-16), (self.y-16))
		end
  end
end

function Thug:updateState(dt)
	local leftBound = self.initx - self.lb
	local rightBound = self.initx + self.rb
	
	--state 0 = wait
	if (self.state == 0) then
		-- stop the thug
		self.x_vel = 0
		-- update movement
		self:updateMvmt(dt)
		
		-- the thug is towards the player and is in the Y LOS
		if self.towards and self:yHit() then
			-- distance is right!
			if dist <= 150 then
				self.state = 2											-- switch to follow state!
			end
		end
		
		-- thug is done waiting
		if self.time >= random_wait_time then
			self.time = 0													-- reset the timer
			-- where is the thug going?
			if self.dir == 1 then
				self.x_vel = thug_walk
			elseif self.dir == 0 then
				self.x_vel = -1*thug_walk
			end
			-- switch to patrol
			self.state = 1
		end
			
	--state 1 = patrol
	elseif (self.state == 1) then
		-- if the thug hits the left boundary
		if (thex < leftBound) and not self.isPush then
			self.x_vel = thug_walk
			self.dir = 1
			self.time = 0											-- for the wait state
			self.state = 0
		-- if the thug hits the right boundary
		elseif thex > rightBound and not self.isPush then
			self.x_vel = -1*thug_walk
			self.dir = 0
			self.time = 0											-- for the wait state
			self.state = 0
		-- if the thug pushes a wall
		elseif self.isPush then
			if self.dir == 0 then
				self.x_vel = thug_walk
				self.dir = 1
				self.time = 0
			else
				self.x_vel = -1*thug_walk
				self.dir = 0
				self.time = 0
			end
			self.isPush = false		-- done pushing
			self.state = 0				-- switch to wait state
		end

		self:updateMvmt(dt)

		-- the thug is towards the player and is in the Y LOS
		if self.towards and self:yHit() then
			-- distance is right!
			if dist <= 150 then
				self.state = 2											-- switch to follow state!
			end
		end
			
			
	--state 2 = follow player
	elseif (self.state == 2) then
		
		-- the distance is close enough for a chase
		if dist <= 150 and self.towards and self:yHit() then
			-- switch to chase speed
			if self.dir == 1 then
				self.x_vel = thug_chase
			elseif self.dir == 0 then
				self.x_vel = -1*thug_chase
			end
			self:updateMvmt(dt)						-- update movement
		
		-- the player is close to the wall and thug is pushing against it
		elseif dist <= 32 and self.isPush then
			if self.dir == 1 then
				self.x_vel = thug_chase
			elseif self.dir == 0 then
				self.x_vel = -1*thug_chase
			end
			self:updateMvmt(dt)
			
		-- hit the player!
		elseif dist <= 64 and ((self.x+16 >= p.x+16) or (self.x+16 <= p.x+16))
			and not self.isPush then
			self.time = 0
			self.state = 3			-- switch to attack state
		
		else -- stop the thug from chasing

			self.initx = self.x
			leftBound = self.initx - self.lb
			rightBound = self.initx + self.rb
			self.x_vel = 0 --stop him!
			self.state = 0
		end
	
	elseif (self.state == 3) then
			-- state 3 = attack
			-- stop the thug
			self.x_vel = 0
			-- where the thug is facing and make player face him (hit animation)
			if self.towards and dir == 1 then
				p.enemy_facing = 1
			else
				p.enemy_facing = 0
			end
			
			if self.time >= 3 then
				p.is_hit = true
				attackDamage(self, p)
				p:hit(dt)
				self.time = 0
			end
			
			-- if player is far enough
			if dist > 64 and not self.isPush then
				self.state = 2									-- back to chasing
			end

		-- end of states
	end
end

function Thug:updateAnim(dt)
		if (self.state == 0) then
			tstand:update(dt)
		elseif (self.state == 1) and (self.dir == 1) then
			twalkr:update(dt)
		elseif (self.state == 1) and (self.dir == 0) then
			twalkl:update(dt)
		elseif (self.state == 2) and (self.dir == 1) then
			twalkr:update(dt)
		elseif (self.state == 2) and (self.dir == 0) then
			twalkl:update(dt)
		elseif (self.state == 3) and (self.dir == 1) then
			twalkr:update(dt)
		elseif (self.state == 3) and (self.dir == 0) then
			twalkl:update(dt)
		end
end

-- START OF PHYSICS
function Thug:collide(event)
	if event == "floor" then
		self.y_vel = 0
		self.standing = true
	end
	if event == "ceiling" then
		self.y_vel = 0
	end
end

function Thug:updateMvmt(dt)
	local halfX = self.w / 2
	local halfY = self.h / 2

	self.y_vel = self.y_vel + (world.gravity * dt)

	self.x_vel = math.floor(math.clamp(self.x_vel, -self.max_x_vel, self.max_x_vel))
	self.y_vel = math.floor(math.clamp(self.y_vel, -self.max_y_vel, self.max_y_vel))
	
	nextY = math.floor(self.y + (self.y_vel*dt))
	if self.y_vel < 0 then
		--collide with tiles
		if not (self:collideTile(map, self.x - halfX, nextY - halfY))
			and not (self:collideTile(map, self.x + halfX - 1, nextY - halfY)) then
			self.y = nextY
			self.standing = false
		else
			self.y = nextY + map.tileHeight - ((nextY - halfY) % map.tileHeight)
			self:collide("ceiling")
		end
	elseif self.y_vel > 0 or nextY then
		if not (self:collideTile(map, self.x-halfX+1, nextY + halfY))
			  and not (self:collideTile(map, self.x + halfX-1, nextY + halfY - 1)) then
				self.y = nextY
				self.standing = false
		else
			self.y = nextY - ((nextY + halfY) % map.tileHeight)
			self:collide("floor")
		end
	end
	
	nextX = math.floor(self.x + (self.x_vel*dt))
	if self.x_vel > 0 or nextX > self.x then
		if not (self:collideTile(map, nextX + halfX, self.y - halfY))
		  and not (self:collideTile(map, nextX + halfX, self.y + halfY - 1)) then
		  self.x = nextX
		  self.isPush = false
		else
		  self.x = nextX - ((nextX + halfX) % map.tileWidth)
		  self.isPush = true
		end
	elseif self.x_vel < 0 or nextX < self.x then
		if not (self:collideTile(map, nextX - halfX, self.y - halfY))
		  and not (self:collideTile(map, nextX - halfX, self.y + halfY - 1)) then
		  self.x = nextX
		  self.isPush = false
		else
		  self.x = nextX + map.tileWidth - ((nextX - halfX) % map.tileWidth)
		  self.isPush = true
		end
	end
	
	self:getPlayerFacing(p)
end

--Checking where the player is facing
function Thug:getPlayerFacing(pl)
  local facing = false
  if (self.x_vel < 0) and (pl.x < self.x) then
    facing = true
  elseif (self.x_vel < 0) and (pl.x > self.x) then
    facing = false
  elseif (self.x_vel > 0) and (pl.x > self.x) then
    facing = true
  elseif (self.x_vel > 0) and (pl.x < self.x) then
    facing = false
  end
  self.towards = facing
end

--The thug is colliding with the tile?
function Thug:collideTile(map, x, y)
  local layer = map.tl["solid"]
  local tileX, tileY = math.floor(x / map.tileWidth), math.floor(y / map.tileHeight)
  local tile = layer.tileData(tileX,tileY)
  return not(tile == nil)
end

--Thug's Y LOC
function Thug:yHit()
  if ((p.y+150) >= (self.y)) and ((p.y) <=(self.y+32)) then
    return true
  else return false
  end
end

--Pytagorian theorem of finding distance
function Thug:curDist(x1,y1,x2,y2)
  local calc_x = math.abs(x1-x2)
  local calc_y = math.abs(y1-y2)
  local calc = math.sqrt((calc_x^2) + (calc_y^2))
  return math.floor(calc)
end