--[[
QUEUE ENGINE 0.5 ~ Main.lua

(C) 2013 Clinton S. Cañalita and others
]]--

-- main stuff
require "draw"
require "update"
require "resources"
require "map"
-- entities
require "player"
require "door"
-- enemy entities
require "enemy/thug"
-- calculations
require "attack"

-- principal requires
ATL = require "AdvTiledLoader"
gamera = require "gamera"
anim8 = require "anim8"
cron = require "cron"

-- some important love functions, reduced.
lg = love.graphics
lk = love.keyboard
la = love.audio

--constants (tweak if necessary)

WALK = 300 -- player walk constant
RUN = 500 -- player run constant

JUMPBOOST = 100 -- jumpboost
WALLJUMP = 600 -- wall jump
WALLJUMP_DECREMENT = 40 -- wall jump steps
STICKSLOWDOWN = 1300 -- decent from sticking

WALLJUMP_VERT = -900 -- vertical speed from wall jumping
JUMP_VERT = -550 --OR -1035

FLY_SPEED = 600 -- vertical speed

INIT_LEVEL = 1
INIT_EXPERIENCE = 1000
INIT_HP = 25
INIT_STR = 5
INIT_DEF = 7
INIT_CHA = 50

ATTACK_QUOTIENT = 10

atk = 0 -- an random constant

animation_state = 1 -- player's animation state

function love.load ()
  -- start the game
  -- constant paths
	mpath = "maps/"
	ipath = "img/"
  
  -- level number (initially 1)
	levelNum = 1

  -- tell ATL where is the map
	ATL.Loader.path = mpath

  -- load the map (Map.lua)
	loadMap(levelNum)

	-- define the world values
	world = 	{
		gravity = 1780,
		ground = 512,
	}

	-- setup the graphical and audio resources
	resourceSet()

	-- define engine vars  
	finishLevel = false

	-- define a random number for the randomizer
  math.randomseed(os.time())
	
  -- this is for the fighting animation
	animclock = cron.after(0.4, stopAttack)
	
end

function love.update (dt)
  -- make your delta-time constant
	if dt > 0.05 then
		dt = 0.05
	end
  
  -- set atk with a random number
	atk = math.random(1,2)
  
  --player's controls
	pControl(dt)
  --player's animation
	pAnimation(dt)
  
	if p.state == "death" then
		p.x = map.startx
		p.y = map.starty
		p.lives = p.lives - 1
		p.state = "stand"
	end
	
	if p.lives == 0 then
		p.alive = false
	end
		
  -- update entities
  -- the player
  p:update(dt, world.gravity)
  
  -- the enemies
	for i,v in ipairs(map.enemies) do
		if v.alive then                 -- this tells if the enemy is alive
      v:update(dt)                  -- then update his stuff
		else              							-- else
			p.critical = false						-- bug: premature critical hit.
			table.remove(map.enemies,i)   -- since he's marked dead
			print "DELETED!"
		end                             -- might as well delete him.
	end
  
  -- commands for our camera
	camera:setScale(2)
	camera:setPosition(p.x, p.y)
end

function love.draw ()
  -- start drawing!
  -- from the camera
  camera:draw(function(l,t,w,h)
		-- draw the map
    map:draw()
    -- draw the doors
    for i,v in ipairs(map.doors) do
      v:draw()
    end
    -- draw the player
    if p.alive then
      playerAnim[animation_state]:draw(pimage, (p.x-16), (p.y-16))
    end
    -- draw the enemies
    for i,v in ipairs(map.enemies) do
      if v.alive then
        v:draw()
      end
    end
		-- end of camera bounds
  end)

  -- UI
  debugText() -- debug text for the player
end

function love.keyreleased(key)
  -- key states
  -- quit game
  if key == "escape" then
        love.event.push("quit")  -- actually causes the app to quit
  end
  -- if the l & r arrows were released, do not let the player move!
	if (key == "left") or (key == "right") then
		p.x_vel = 0
	end
  -- the use button
  if (key == "up") then
    for i,v in ipairs(map.doors) do
      if v:collidePlayer(p) and not(v.isOn) then
        v.isOn = true
				print "door is open"
				break
      end
			if v:collidePlayer(p) and v.isOn then
				v:openDoor(p)
				print "going in..."
				break
			end
    end
  end
	-- the attack button
	if (key == "z") then
		p:attack()
		p.atk_count = atk -- capture the number
	end
  
  -- debug keys:
  
	-- reset the game
  if (key == "f11") then
    --remove the player
    table.remove(Player,1)
    --load map again
    loadMap(levelNum)
  end
  
  -- previous map
  if (key == "f9") then
    if levelNum > 1 then
      levelNum = levelNum - 1
      table.remove(Player,1)
      loadMap(levelNum)
    end
  end
  
  -- next map
  if (key == "f10") then
    if levelNum < 4 then
      levelNum = levelNum + 1
      table.remove(Player,1)
      loadMap(levelNum)
    end
  end  
end

-- for the attack animation
function stopAttack()
	p.atk_trigger = false
  -- stop the triggering of attacks
	if animation_state >= 14 or animation_state <= 19 then
		playerAnim[animation_state]:gotoFrame(1)
	end
  -- reset the clock
	p.will_hit = true
	animclock:reset()
end

function math.clamp(x, min, max)
  return x < min and min or (x > max and max or x)
end