--[[
QUEUE ENGINE 0.5 ~ Player.lua

(C) 2013 Clinton S. Cañalita and others
]]--

Player = 	{}

function Player:new()
  local object = {
      alive = true, -- player is alive?
      -- coordinates and basic attributes
      x = 256,
      y = 256,
      x_vel = 0,
      y_vel = 0,
      h = 32,
      w = 16,
      state = "",
      standing = false,
      lives = 3,
      health = INIT_HP,
      level = INIT_LEVEL,
			--exp = 0,
			--total_exp = INIT_EXPERIENCE,
			str = INIT_STR,
			def = INIT_DEF,
			cha = INIT_CHA,
			
      -- player (so-called) physics
      jump_vel = JUMP_VERT,           -- jump velocity
      wall_jump_vel = WALLJUMP_VERT,  -- wall jump velocity
      speed = WALK,                   -- player's speed (walk or run)
      sp_status = "walk",             -- player's speed's status
      flySpeed = FLY_SPEED,           -- player's fly speed
      y_stick = 0,                    -- player's y_vel during sticking
      side_stick = 0,                 -- the player's side sticking direction
      hor_jump_vel = 0,               -- the horizontal jumping velocity
      bJumpLetGo = true,
			
      -- other junk
      time = 0,                       -- player's hardcoded time
      isPush = false,                 -- is player pushing a wall?!
      dir = 1, -- 0 = left, 1 = right
			
      -- fighting system
      atk_trigger = false,            -- pressed attack?
			is_hit = false,									-- is the player hit?
			will_hit = false,               -- is the player hitting?
      combo_count = 0,                -- combo counter
      critical = false,               -- critical!
			enemy_facing = 0,               -- where the enemy is facing
			
      time = 0                        -- time
    }
    
    setmetatable(object, { __index = Player })
    return object
end

function Player:jump()
		if self.standing and self.bJumpLetGo then
			self.y_vel = self.jump_vel
			self.speed = self.speed + JUMPBOOST
			self.standing = false
			self.bJumpLetGo = false
		elseif not(self.side_stick == 0) and self.bJumpLetGo then
			self.y_vel = self.wall_jump_vel
			if self.sp_status == "run" then
				self.speed = RUN + JUMPBOOST
			else
				self.speed = WALK + JUMPBOOST
			end
			if self.side_stick == -1 then
				self.hor_jump_vel = WALLJUMP
			elseif self.side_stick == 1 then
				self.hor_jump_vel = -1*WALLJUMP
			end
			self.standing = false
			self.bJumpLetGo = false
		end
end

function Player:jumpLetGo()
	if self.standing or not(self.side_stick == 0) then
		self.bJumpLetGo = true
	end
end

function Player:right()
	if self.y_stick == 0 then
		self.x_vel = self.speed
	end
	self.dir = 1
end

function Player:left()
	if self.y_stick == 0 then
		self.x_vel = -1*self.speed
	end
	self.dir = 0
end

function Player:stop()
	self.x_vel = 0
end

function Player:walk()
	if self.standing then
		self.speed = WALK
	end
	self.sp_status = "walk"
end

function Player:run()
	if self.standing then
		self.speed = RUN
	end
	self.sp_status = "run"
end
  
function Player:collide(event)
	if event == "floor" then
		self.y_vel = 0
		self.standing = true
	end
	if event == "ceiling" then
		self.y_vel = 0
	end
end

function Player:attack()
	-- player has pressed attack
	self.atk_trigger = true
	if animation_state >= 14 or animation_state <= 19 then
		playerAnim[animation_state]:resume()
	end
end

function Player:hit(dt)
	-- player is hit
	if self.is_hit then
		self.y_vel = -400
		self.x = self.x - dt*2
	end
end

function Player:update(dt)

	local halfX = self.w / 2
	local halfY = self.h / 2

	self.y_vel = self.y_vel + ((world.gravity - self.y_stick) * dt)

	if self.standing then
		self.hor_jump_vel = 0
	elseif self.hor_jump_vel > 0 then
		if self.x_vel > 0 then
			self.hor_jump_vel = 0
		else
			self.hor_jump_vel = self.hor_jump_vel - WALLJUMP_DECREMENT
		end
	elseif self.hor_jump_vel < 0 then
		if self.x_vel < 0 then
			self.hor_jump_vel = 0
		else
			self.hor_jump_vel = self.hor_jump_vel + WALLJUMP_DECREMENT
		end
	end

	self.x_vel = math.floor(math.clamp(self.x_vel, -self.speed, self.speed))
	self.y_vel = math.floor(math.clamp(self.y_vel, -self.flySpeed, self.flySpeed))

	local nextY = self.y + (self.y_vel*dt)
	local nextX
	
	if self.hor_jump_vel == 0 then
		nextX = self.x + (self.x_vel*dt)
	else
		nextX = self.x + (self.hor_jump_vel*dt)
	end

	if self.y_vel < 0 then
		if not (self:isColliding(map,self.x-halfX+1, nextY - halfY))
			and not (self:isColliding(map,self.x + halfX - 1, nextY - halfY)) then
			self.y = nextY
			self.standing = false
		else
			self.y = nextY + map.tileHeight - ((nextY - halfY) % map.tileHeight)
			self:collide("ceiling")
		end
		self.y_stick = 0
	elseif self.y_vel > 0 then
		if not (self:isColliding(map, self.x-halfX+1, nextY + halfY))
			and not (self:isColliding(map, self.x+halfX-1,nextY+halfY)) then
			self.y = nextY
			self.standing = false
			if (self:isColliding(map, self.x + halfX, self.y)) then
				self.side_stick = 1
				self.y_stick = STICKSLOWDOWN
			elseif (self:isColliding(map, self.x - halfX-1, self.y)) then
				self.side_stick = -1
				self.y_stick = STICKSLOWDOWN
			else
				self.side_stick = 0
				self.y_stick = 0
			end
		else
			self.y = nextY - ((nextY + halfY) % map.tileHeight)
			self:collide("floor")
			self.side_stick = 0
			self.y_stick = 0
		end
	end
	
	if self.x_vel > 0 or nextX > self.x then
		if not (self:isColliding(map, nextX + halfX, self.y))
			and not (self:isColliding(map, nextX + halfX, self.y + halfY - 1)) then
			self.x = nextX
			self.isPush = false
		else
			self.x = nextX - ((nextX + halfX) % map.tileWidth)
			self.isPush = true
		end
	elseif self.x_vel < 0 or nextX < self.x then
		if not (self:isColliding(map, nextX - halfX, self.y))
			and not (self:isColliding(map, nextX - halfX, self.y + halfY - 1)) then
			self.x = nextX
			self.isPush = false
		else
			self.x = nextX + map.tileWidth - ((nextX - halfX) % map.tileWidth)
			self.isPush = true
		end
	end

	--reseting combos
	if self.combo_count > 0 and self.atk_trigger == false then
		self.time = self.time + dt*2
		if self.time >= math.pi then
			self.time = 0
			self.combo_count = 0
		end
	end

	self.state = self:getState()
end

function Player:isColliding(map, x, y)
	local layer = map.tl["solid"]
	local tileX, tileY = math.floor(x / map.tileWidth), math.floor(y / map.tileHeight)
	local tile = layer.tileData(tileX,tileY)
	return not(tile == nil)
end

function Player:getState()
	local tempState = ""
	
	if self.standing then
		if self.x_vel > 0 then
			tempState = "right"
		elseif self.x_vel < 0 then
			tempState = "left"
		else
			tempState = "stand"
		end
	end
	
	if self.y_vel < 0 then
		tempState = "jump"
	elseif self.y_vel > 0 then
		tempState = "fall"
	end
	
	if (self.y > (map.height * map.tileHeight) + self.h) then
	tempState = "death"
	end
	
	if (self.health <= 0) then
	tempState = "death"
	end
	
	if not (self.side_stick == 0) then
	tempState = "slide"
	end
	
	if self.atk_trigger then
	tempState = "attack"
	end
	
	return tempState
end